package com.hzs.member.statis.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.hzs.common.domain.member.statis.CuBonusVertexStatis;
import com.hzs.member.statis.vo.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 奖金拨比零线统计 Mapper 接口
 */
public interface CuBonusVertexStatisMapper extends BaseMapper<CuBonusVertexStatis> {

    /**
     * 奖金拨比列表
     **/
    List<CuBonusVertexStatisVO> queryCuBonusStatis(CuBonusVertexStatisVO cuBonusVertexStatisVO);

    /**
     * 查询奖金拨比列表（PV）
     **/
    List<CuBonusVertexStatisPvVO> queryCuBonusPvStatis(CuBonusVertexStatisPvVO cuBonusVertexStatisPvVO);

    /**
     * 查询顶点奖金拨比汇总
     **/
    List<CuBonusVertexStatisTotalVO> queryCuBonusStatisTotal(CuBonusVertexStatisTotalVO cuBonusVertexStatisTotalVO);

    /**
     * 顶点奖金拨比汇总查询总和
     **/
    CuBonusVertexStatisTotalVO queryCuBonusStatisSum(CuBonusVertexStatisTotalVO cuBonusVertexStatisTotalVO);

    /**
     * 查询时间内奖金拨比顶点会员
     **/
    List<CuBonusVertexStatisTotalVO> queryCuBonusVertex(CuBonusVertexStatisTotalVO cuBonusVertexStatisTotalVO);

    /**
     * 查询顶点拓展封顶人数
     **/
    CuBonusVertexStatisTotalVO queryOrgTopNumber(CuBonusVertexStatisVO cuBonusVertexStatisVO);

    /**
     * 查询顶点拓展平均碰次和拓展最高碰次
     **/
    CuBonusVertexStatisTotalVO queryTouch(CuBonusVertexStatisVO cuBonusVertexStatisVO);

    /**
     * 奖金拨比列表统计小计（最后一行）
     **/
    CuBonusVertexStatisVO queryCuBonusSumStatis(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * 奖金拨比列表PV统计小计（最后一行）
     **/
    CuBonusVertexStatisPvVO queryCuBonusPvSumStatis(CuBonusVertexStatisPvVO cuBonusVertexStatisPvVO);

    /**
     * 7天重算查询订单
     **/
    List<CuBonusVertexStatisVO> queryAnewSaOrderByLessDay(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * 删除奖金拨比统计表数据
     **/
    Boolean delStatis(@Param("startDate") String startDate,
                      @Param("endDate") String endDate,
                      @Param("pkVertexList") List<Integer> pkVertexList);

    /**
     * 查询中国奖金
     **/
    List<CuBonusVertexStatisVO> queryChinaCountryBonus(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * 查询7天重算奖金查询
     **/
    List<CuBonusVertexStatisVO> queryVertexChinaCountryBonus(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * 查询生效国外国家列表
     **/
    List<CuBonusVertexStatisVO> countrylist(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * 查询当天团队统计数据
     **/
    List<CuBonusVertexStatis> getCuBonusStatis(CuBonusVertexStatis CuBonusVertexStatis);

}
